package com.sun.imageio.plugins.jpeg;

import javax.imageio.stream.ImageOutputStream;
import org.w3c.dom.NamedNodeMap;
import javax.imageio.metadata.IIOInvalidTreeException;
import javax.imageio.metadata.IIOMetadataNode;
import org.w3c.dom.Node;
import java.io.IOException;
import javax.imageio.IIOException;

class MarkerSegment implements Cloneable
{
    protected static final int LENGTH_SIZE = 2;
    int tag;
    int length;
    byte[] data;
    boolean unknown;
    
    MarkerSegment(JPEGBuffer jpegBuffer) throws IOException {
        this.data = null;
        this.unknown = false;
        jpegBuffer.loadBuf(3);
        this.tag = (jpegBuffer.buf[jpegBuffer.bufPtr++] & 0xFF);
        this.length = (jpegBuffer.buf[jpegBuffer.bufPtr++] & 0xFF) << 8;
        this.length |= (jpegBuffer.buf[jpegBuffer.bufPtr++] & 0xFF);
        this.length -= 2;
        if (this.length < 0) {
            throw new IIOException("Invalid segment length: " + this.length);
        }
        jpegBuffer.bufAvail -= 3;
        jpegBuffer.loadBuf(this.length);
        return;
    }
    
    MarkerSegment(int tag) {
        this.data = null;
        this.unknown = false;
        this.tag = tag;
        this.length = 0;
    }
    
    MarkerSegment(Node node) throws IIOInvalidTreeException {
        this.data = null;
        this.unknown = false;
        this.tag = getAttributeValue(node, null, "MarkerTag", 0, 255, true);
        this.length = 0;
        if ((node instanceof IIOMetadataNode)) {
            IIOMetadataNode iioMetadataNode = (IIOMetadataNode)node;
            try {
                this.data = (byte[])(byte[])iioMetadataNode.getUserObject();
            }
            catch (Exception cause) {
                IIOInvalidTreeException ex = new IIOInvalidTreeException("Can't get User Object", node);
                ex.initCause(cause);
                throw ex;
            }
            return;
        }
        throw new IIOInvalidTreeException("Node must have User Object", node);
    }
    
    @Override
    protected Object clone() {
        MarkerSegment markerSegment = null;
        try {
            markerSegment = (MarkerSegment)super.clone();
        }
        catch (CloneNotSupportedException ex) {}
        if (this.data != null) {
            markerSegment.data = (byte[])(byte[])this.data.clone();
            return markerSegment;
        }
        return markerSegment;
    }
    
    void loadData(JPEGBuffer jpegBuffer) throws IOException {
        jpegBuffer.readData(this.data = new byte[this.length]);
    }
    
    IIOMetadataNode getNativeNode() {
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("unknown");
        iioMetadataNode.setAttribute("MarkerTag", Integer.toString(this.tag));
        iioMetadataNode.setUserObject(this.data);
        return iioMetadataNode;
    }
    
    static int getAttributeValue(Node node, NamedNodeMap attributes, String s, int n, int n2, boolean b) throws IIOInvalidTreeException {
        if (attributes == null) {
            attributes = node.getAttributes();
        }
        String nodeValue = attributes.getNamedItem(s).getNodeValue();
        int int1 = -1;
        if (nodeValue == null) {
            if (b) {
                throw new IIOInvalidTreeException(s + " attribute not found", node);
            }
        }
        else {
            int1 = Integer.parseInt(nodeValue);
            if (int1 < n || int1 > n2) {
                throw new IIOInvalidTreeException(s + " attribute out of range", node);
            }
        }
        return int1;
    }
    
    void writeTag(ImageOutputStream imageOutputStream) throws IOException {
        imageOutputStream.write(255);
        imageOutputStream.write(this.tag);
        write2bytes(imageOutputStream, this.length);
    }
    
    void write(ImageOutputStream imageOutputStream) throws IOException {
        this.length = 2 + ((this.data != null) ? this.data.length : 0);
        this.writeTag(imageOutputStream);
        if (this.data != null) {
            imageOutputStream.write(this.data);
        }
    }
    
    static void write2bytes(ImageOutputStream imageOutputStream, int n) throws IOException {
        imageOutputStream.write(n >> 8 & 0xFF);
        imageOutputStream.write(n & 0xFF);
    }
    
    void printTag(String str) {
        System.out.println(str + " marker segment - marker = 0x" + Integer.toHexString(this.tag));
        System.out.println("length: " + this.length);
    }
    
    void print() {
        this.printTag("Unknown");
        if (this.length > 10) {
            System.out.print("First 5 bytes:");
            for (int j = 0; j < 5; ++j) {
                System.out.print(" Ox" + Integer.toHexString((int)this.data[j]));
            }
            System.out.print("\nLast 5 bytes:");
            for (int k = this.data.length - 5; k < this.data.length; ++k) {
                System.out.print(" Ox" + Integer.toHexString((int)this.data[k]));
            }
        }
        else {
            System.out.print("Data:");
            for (int i = 0; i < this.data.length; ++i) {
                System.out.print(" Ox" + Integer.toHexString((int)this.data[i]));
            }
        }
        System.out.println();
    }
}
